@@ -35,7 +35,7 @@ class ExpressionMakeCoroutineObject(ExpressionChildrenHavingBase):
 
     getCoroutineRef = ExpressionChildrenHavingBase.childGetter("coroutine_ref")
 
-    def __init__(self, coroutine_ref, code_object, source_ref):
+    def __init__(self, coroutine_ref, source_ref):
         assert coroutine_ref.getFunctionBody().isExpressionCoroutineObjectBody()
 
         ExpressionChildrenHavingBase.__init__(
@@ -46,18 +46,8 @@ class ExpressionMakeCoroutineObject(ExpressionChildrenHavingBase):
             source_ref = source_ref
         )
 
-        self.code_object = code_object
-
         self.variable_closure_traces = None
 
-    def getDetails(self):
-        return {
-            "code_object" : self.code_object
-        }
-
-    def getCodeObject(self):
-        return self.code_object
-
     def getDetailsForDisplay(self):
         return {
             "coroutine" : self.getCoroutineRef().getFunctionBody().getCodeName()
@@ -92,11 +82,12 @@ class ExpressionCoroutineObjectBody(ExpressionFunctionEntryPointBase):
 
     qualname_setup = None
 
-    def __init__(self, provider, name, flags, source_ref):
+    def __init__(self, provider, name, code_object, flags, source_ref):
         ExpressionFunctionEntryPointBase.__init__(
             self,
             provider    = provider,
             name        = name,
+            code_object = code_object,
             code_prefix = "coroutine",
             flags       = flags,
             source_ref  = source_ref
@@ -156,6 +147,9 @@ class ExpressionAsyncWait(ExpressionChildrenHavingBase):
 
     def computeExpression(self, trace_collection):
         # TODO: Might be predictable based awaitable analysis or for constants.
+
+        trace_collection.onExceptionRaiseExit(BaseException)
+
         return self, None, None
 
     getValue = ExpressionChildrenHavingBase.childGetter("expression")
